﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectFighterJet;

/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawningFighterJet
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityFighterJet? EntityFighterJet { get; private set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки истрибителя
    /// </summary>
    private int? _startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки истрибителя
    /// </summary>
    private int? _startPosY;
    /// <summary>
    /// Ширина прорисовки истрибителя
    /// </summary>
    private readonly int _drawningJetrWidth = 135;
    /// <summary>
    /// Высота прорисовки истрибителя
    /// </summary>
    private readonly int _drawningJetrHeight = 135;

    /// <summary>
    /// Инициализация свойств
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="rockets">Признак наличия ракет</param>
    /// <param name="fuel">Признак наличия топливного бака</param>
    /// <param name="engines">Признак наличия двигателей</param>
    public void Init(int speed, double weight, Color bodyColor, Color
    additionalColor, bool rockets, bool fuel, bool engines)
    {
        EntityFighterJet = new EntityFighterJet();
        EntityFighterJet.Init(speed, weight, bodyColor, additionalColor,
        rockets, fuel, engines);
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }
    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        // TODO проверка, что объект "влезает" в размеры поля
        // если влезает, сохраняем границы и корректируем позицию объекта, если она была уже установлена
        if (width < _drawningJetrWidth || height < _drawningJetrHeight) return false;
        _pictureWidth = width;
        _pictureHeight = height;
        if (_startPosX != null && _startPosY != null)
        {
            if (_startPosX + _drawningJetrWidth > _pictureWidth)
            {
                _startPosX = -_drawningJetrWidth + _pictureWidth;
            }
            else if (_startPosX < 0)
            {
                _startPosX = 0;
            }
            if (_startPosY + _drawningJetrHeight > _pictureHeight)
            {
                _startPosY = -_drawningJetrHeight + _pictureHeight;
            }
            else if (_startPosY < 0)
            {
                _startPosY = 0;
            }
        }
        return true;
        
    }
    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        // TODO если при установке объекта в эти координаты, он будет "выходить" за границы формы
        // то надо изменить координаты, чтобы он оставался в этих границах
        if (x + _drawningJetrWidth > _pictureWidth)
        {
            _startPosX = x - (x + _drawningJetrWidth - _pictureWidth);
        }
        else if (x < 0)
        {
            _startPosX = 0;
        }
        else
        {
            _startPosX = x;
        }
        if (y + _drawningJetrHeight > _pictureHeight)
        {
            _startPosY = y - (y + _drawningJetrHeight - _pictureHeight);
        }
        else if (y < 0)
        {
            _startPosY = 0;
        }
        else
        {
            _startPosY = y;
        }
        _startPosX = x;
        _startPosY = y;
    }

    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityFighterJet == null || !_startPosX.HasValue ||
        !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityFighterJet.Step > 0)
                {
                    _startPosX -= (int)EntityFighterJet.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityFighterJet.Step > 0)
                {
                    _startPosY -= (int)EntityFighterJet.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX + _drawningJetrWidth + EntityFighterJet.Step < _pictureWidth)
                {
                    _startPosX += (int)EntityFighterJet.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY + _drawningJetrHeight + EntityFighterJet.Step < _pictureHeight)
                {
                    _startPosY += (int)EntityFighterJet.Step;
                }
                return true;
            default:
                return false;
        }   
    }

    public void DrawTransport(Graphics g)
    {
        if (EntityFighterJet == null || !_startPosX.HasValue ||
        !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);
        Brush additionalBrush = new
        SolidBrush(EntityFighterJet.AdditionalColor);
        Brush brBlue = new SolidBrush(Color.LightBlue);
        //Нос бомбардировщика
        g.DrawLine(pen, _startPosX.Value + 20, _startPosY.Value + 60, _startPosX.Value + 10, _startPosY.Value + 70);
        g.DrawLine(pen, _startPosX.Value + 10, _startPosY.Value + 70, _startPosX.Value + 20, _startPosY.Value + 80);
        //топливные баки
        if (EntityFighterJet.Fuel)
        {
            g.FillRectangle(additionalBrush, _startPosX.Value + 80, _startPosY.Value + 50, 30, 10);
            g.FillRectangle(additionalBrush, _startPosX.Value + 80, _startPosY.Value + 80, 30, 10);
        }
        //Тело бомбардировщика
        brBlue = new SolidBrush(Color.White);
        g.DrawRectangle(pen, _startPosX.Value + 20, _startPosY.Value + 60, 120, 20);
        //Задние крылья бомбардировщика
        g.DrawLine(pen, _startPosX.Value + 140, _startPosY.Value + 30, _startPosX.Value + 140, _startPosY.Value + 110);
        g.DrawLine(pen, _startPosX.Value + 140, _startPosY.Value + 110, _startPosX.Value + 120, _startPosY.Value + 90);
        g.DrawLine(pen, _startPosX.Value + 120, _startPosY.Value + 90, _startPosX.Value + 120, _startPosY.Value + 80);
        g.DrawLine(pen, _startPosX.Value + 140, _startPosY.Value + 30, _startPosX.Value + 120, _startPosY.Value + 50);
        g.DrawLine(pen, _startPosX.Value + 120, _startPosY.Value + 50, _startPosX.Value + 120, _startPosY.Value + 60);
        //Крылья бомбардировщика
        g.DrawLine(pen, _startPosX.Value + 50, _startPosY.Value, _startPosX.Value + 50, _startPosY.Value + 60);
        g.DrawLine(pen, _startPosX.Value + 50, _startPosY.Value + 80, _startPosX.Value + 50, _startPosY.Value + 135);
        g.DrawLine(pen, _startPosX.Value + 50, _startPosY.Value + 135, _startPosX.Value + 60, _startPosY.Value + 135);
        g.DrawLine(pen, _startPosX.Value + 60, _startPosY.Value + 135, _startPosX.Value + 65, _startPosY.Value + 80);
        g.DrawLine(pen, _startPosX.Value + 50, _startPosY.Value, _startPosX.Value + 60, _startPosY.Value);
        g.DrawLine(pen, _startPosX.Value + 60, _startPosY.Value, _startPosX.Value + 65, _startPosY.Value + 60);
       



        //Ракеты бомбардировщика
        if (EntityFighterJet.Rockets)
        {
            g.FillRectangle(additionalBrush, _startPosX.Value + 35, _startPosY.Value + 10, 15, 5);
            g.FillRectangle(additionalBrush, _startPosX.Value + 35, _startPosY.Value + 125, 15, 5);
            g.FillRectangle(additionalBrush, _startPosX.Value + 35, _startPosY.Value + 50, 15, 5);
            g.FillRectangle(additionalBrush, _startPosX.Value + 35, _startPosY.Value + 85, 15, 5);

        }
        //Двигатели
        if (EntityFighterJet.Engines)
        {
            g.FillRectangle(additionalBrush, _startPosX.Value + 40, _startPosY.Value + 35, 10, 10);
            g.FillRectangle(additionalBrush, _startPosX.Value + 40, _startPosY.Value + 95, 10, 10);
            g.FillRectangle(additionalBrush, _startPosX.Value + 40, _startPosY.Value + 20, 10, 10);
            g.FillRectangle(additionalBrush, _startPosX.Value + 40, _startPosY.Value + 110, 10, 10);
        }

    }
}


